/**
 * Copyright (c) 2009-2012 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.zauberlabs.commons.mom.internal;

import java.lang.reflect.InvocationTargetException;
import java.util.Map;
import java.util.Map.Entry;

import net.sf.staccatocommons.check.Ensure;
import net.sf.staccatocommons.lang.SoftException;
import net.sf.staccatocommons.restrictions.check.NonNull;
import net.sf.staccatocommons.restrictions.processing.EnforceRestrictions;

import com.zauberlabs.commons.mom.AbstractMapObjectMapper;
import com.zauberlabs.commons.mom.MapObjectMapper;
import com.zauberlabs.commons.mom.PropertyModel;
import com.zauberlabs.commons.mom.converter.TypeConverter;

/**
 * {@link MapObjectMapper}
 *
 * @author flbulgarelli
 */
@SuppressWarnings("unchecked")
@EnforceRestrictions
public class DefaultMapObjectMapper extends AbstractMapObjectMapper {

    private final MappingPackage mappingPackage;
    private final TypeConverter typeConverter;
    private final PropertyModel propertyModel;

    /**
     * Creates the {@link DefaultMapObjectMapper
     *
     * @param packageName the name of the mapping packing.
     *        Only objects under the given package will
     *        be considered as composite data type
     * @param typeConverter the type converter used to perform the type coercions
     * @see com.zauberlabs.commons.mom.MapObjectMapper#map(Object)
     */
    public DefaultMapObjectMapper(@NonNull final String packageName,
            @NonNull final PropertyModel propertyModel, @NonNull final TypeConverter typeConverter) {
        this.mappingPackage = new MappingPackage(packageName);
        this.typeConverter = typeConverter;
        this.propertyModel = propertyModel;
    }

    @Override
    public boolean isCompositeType(@NonNull final Class<?> clazz) {
        return mappingPackage.isInMappeablePackage(clazz);
    }

    @Override
    protected Map<String, Object> mapObjectImpl(final Object value) {
        Ensure.that(this.isCompositeType(value.getClass()), "%s must be a composite type", value);
        return new AttributesMap(value, this);
    }

    @Override
    protected Object coerce(final Object value, final Class<?> type) {
        return typeConverter.convert(value, type);
    }

    @Override
    protected <A> A unmapMapImpl(final Map<String, Object> map, final Class<A> targetType) {
        try {
            Object bean = targetType.newInstance();
            unmapProperties(map, bean);
            return (A) bean;
        } catch (Exception e) {
            throw SoftException.soften(e);
        }
    }

    /**
     * Transform each entry value of the map into the object field with the same
     * name of the entry key.
     *
     * @param source
     *            Map with the fields to transform.
     * @param destination
     *            Object to be filled.
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     */
    protected void unmapProperties(@NonNull final Map<String, Object> source,
            @NonNull final Object destination) throws IllegalAccessException, InvocationTargetException {
        for (Entry<String, Object> entry : source.entrySet()) {
            if (!entry.getKey().equals(JAVA_TYPE_KEY)) {
                propertyModel.setProperty(entry.getValue(), destination, entry.getKey(), this);
            }
        }
    }

}
